/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#ifndef _CONNECTION_H
#define _CONNECTION_H

#include "../config.h"
#include <DSNetwork.h>
#include <DSList.h>

#ifndef __FILE__
#define __FILE__ ""
#endif

#include "AddressInfo.h"

class Connection {
	protected:
	DSNetwork * net;
	char * buffer; /* A pointer to where to read/write */
	int bufflen;   /* How big is that buffer in the window ... */
	int buff_upto; /* Where are we up to in it */

	char * user;

	AddressInfo source;
	AddressInfo dest;
	AddressInfo source_on_server;

	unsigned long nExternalSendBytes;
	unsigned long nExternalRecvBytes;
	public:
	/* This constructor associates with a DSNetwork for TCP connections */
	Connection(DSNetwork * thenet);

	/* This constructor associates with a UDP packet */
	Connection(SOCKADDR * sa, char * udppacket, int udplen);
	~Connection();

	unsigned long getDownloadBytes();
	unsigned long getUploadBytes();

	BOOL AllowConnection();
	BOOL IsAllowed(int type);
	BOOL ChildThread();

	BOOL sendData(const char * szData, int nSize=-1);
	BOOL getChar(unsigned char * ch);
	BOOL getChar(char * ch);
	BOOL getSlab(char * buffer, int nSize);

	BOOL sendResponsev4(char rep, unsigned long addr,
			unsigned short port);
	SOCKET getSocket();

	BOOL forwardData(SOCKET two);
	BOOL log(int);
	char * getRemainingData();
	int getRemainingLength();
	char * getCompleteData();
	int getCompleteLength();

	char * getSourceAddress();
	int getSourceAddressLength();
	unsigned int getSourcePort();
	int getSourceAddressType();
	char * getSourceString();
	char * getSourceAddrString();
	BOOL getSourceSockaddr(SOCKADDR **, sl_t *);

	char * getSrvSourceAddress();
	int getSrvSourceAddressLength();
	unsigned int getSrvSourcePort();
	int getSrvSourceAddressType();
	char * getSrvSourceString();
	char * getSrvSourceAddrString();
	BOOL getSrvSourceSockaddr(SOCKADDR **, sl_t *);

	char * getDestAddress();
	int getDestAddressLength();
	unsigned int getDestPort();
	int getDestAddressType();
	BOOL getDestNullAddr();
	char * getDestString();
	char * getDestAddrString();
	BOOL getDestSockaddr(SOCKADDR **, sl_t *);

	void setDestAddress(char *, int);
	BOOL setDestHostname(char *);
	void setDestPort(unsigned int);
	void setDestAddressType(int);

	char * getUser();
	void setUser(char *);

};

void * ChildThread(void * param);

#endif

