/* ANTINAT
 * =======
 * This software is Copyright (c) 2003 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../config.h"
#include <sys/types.h>
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#include "../address.h"
#include "../AddressInfo.h"
#include <DSNetwork.h>

BOOL ipv4_fromsockaddr(AddressInfo * dest, SOCKADDR_IN * sin)
{
	dest->setAddress((char *)&sin->sin_addr.s_addr,4);
	dest->setPort(sin->sin_port);
	dest->setAddressType(AF_INET);
	return TRUE;
}

BOOL ipv4_tosockaddr(AddressInfo * src, SOCKADDR_IN ** ret, sl_t * len)
{
	SOCKADDR_IN * sin;
	sin=(SOCKADDR_IN *)malloc(sizeof(SOCKADDR_IN));
	memset(sin,0,sizeof(SOCKADDR_IN));
	sin->sin_family=src->getAddressType();
	sin->sin_port=src->getPort();
	memcpy(&sin->sin_addr.s_addr, src->getAddress(), 4);
	*ret=sin;
	*len=sizeof(SOCKADDR_IN);
	return TRUE;
}

char * ipv4_getaddrstring(AddressInfo * src)
{
	char * str;
	unsigned long addr;
	memcpy(&addr,src->getAddress(),4);
	addr=ntohl(addr);
	str=(char *)malloc(25);
	if (str!=NULL) {
		sprintf(str,"%lu.%lu.%lu.%lu",
				addr/0x01000000,
				(addr&0xffffff)/0x010000,
				(addr&0xffff)/0x0100,
				(addr&0xff));
	}
	return str;
}

char * ipv4_getstring(AddressInfo * src)
{
	char * str;
	unsigned short port;
	char tmp[10];
	str=ipv4_getaddrstring(src);
	if (str!=NULL) {
		port=src->getPort();
		port=ntohs(port);
		sprintf(tmp,"%i",port);
		strcat(str,":");
		strcat(str,tmp);
	}
	return str;
}

extern "C" {

int libipv4_addr_address_type()
{
	return AF_INET;
}

void * libipv4_addr_getfunction(int func)
{
	switch(func) {
		case FUNC_FROMSOCKADDR:
			return (void *)ipv4_fromsockaddr;
		case FUNC_TOSOCKADDR:
			return (void *)ipv4_tosockaddr;
		case FUNC_GETSTRING:
			return (void *)ipv4_getstring;
		case FUNC_GETADDRSTRING:
			return (void *)ipv4_getaddrstring;
		default:
			return NULL;
	}
}
}
