/* ANTINAT
 * =======
 * This software is Copyright (c) 2003 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../config.h"
#include <sys/types.h>
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#include "../Shared.h"
#include "../address.h"
#include "../AddressInfo.h"
#include <DSNetwork.h>

BOOL ipv6_fromsockaddr(AddressInfo * dest, SOCKADDR_IN6 * sin)
{
	dest->setAddress((char *)&sin->sin6_addr.s6_addr,16);
	dest->setPort(sin->sin6_port);
	dest->setAddressType(AF_INET);
	return TRUE;
}

BOOL ipv6_tosockaddr(AddressInfo * src, SOCKADDR_IN6 ** ret, sl_t * len)
{
	SOCKADDR_IN6 * sin;
	sin=(SOCKADDR_IN6 *)malloc(sizeof(SOCKADDR_IN6));
	memset(sin,0,sizeof(SOCKADDR_IN6));
	sin->sin6_family=src->getAddressType();
	sin->sin6_port=src->getPort();
	memcpy(sin->sin6_addr.s6_addr, src->getAddress(), 16);
	*ret=sin;
	*len=sizeof(SOCKADDR_IN6);
	return TRUE;
}

char * ipv6_getaddrstring(AddressInfo * src)
{
	char * str;
	char addr[18];
	int i;
	memcpy(addr,src->getAddress(),16);
	str=(char *)malloc(80);
	if (str!=NULL) {
		char szTemp[6];
		strcpy(str,"");
		for (i=0;i<16;i++) {
			if (i>0) strcat(str,":");
			sprintf(szTemp,"%x",addr[i]);
			strcat(str,szTemp);
		}
	}
	return str;
}

char * ipv6_getstring(AddressInfo * src)
{
	char * str;
	unsigned short port;
	char tmp[10];
	str=ipv6_getaddrstring(src);
	if (str!=NULL) {
		port=src->getPort();
		port=ntohs(port);
		sprintf(tmp,"%i",port);
		strcat(str,":");
		strcat(str,tmp);
	}
	return str;
}

extern "C" {

int libipv6_addr_address_type()
{
	return AF_INET6;
}

void * libipv6_addr_getfunction(int func)
{
	switch(func) {
		case FUNC_FROMSOCKADDR:
			return (void *)ipv6_fromsockaddr;
		case FUNC_TOSOCKADDR:
			return (void *)ipv6_tosockaddr;
		case FUNC_GETSTRING:
			return (void *)ipv6_getstring;
		case FUNC_GETADDRSTRING:
			return (void *)ipv6_getaddrstring;
		default:
			return NULL;
	}
}
}
