/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../config.h"
#include <DSThread.h>
#include "Connection.h"
#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <DSNetwork.h>
#include <DSComponent.h>
#include <DSConfigFile.h>
#include <DSList.h>
#include <DSParams.h>
#include "loggers.h"
#include "moddecl.h"


DSNetwork server;
DSHashTable versions;
DSList filters;
DSList loggers;
DSHashTable addresses;
DSConfigFile config;

/*
This big messy function processes each incoming request.
*/
BOOL
HandleRequest ()
{
	DSNetwork * net;
	Connection * conn;
	DSThread thr;

	/* Accept an incoming request, or wait till one arrives. */
	net=server.acceptIncoming();
	if (!net) return FALSE;
	net->linger();
	conn=new Connection(net);

	thr.exec(ChildThread, conn);
	thr.detach();

	return TRUE;

}

BOOL logEvent(int event)
{
	DSListElement * le;
	proc_logger logger;
	for (le=loggers.getHead();le!=NULL;le=le->getNext()) {
		logger=(proc_logger)le->getDataPtr();
		logger(NULL,event);
	}
	return TRUE;
}

void daemonize()
{
	int fh;
	if (fork()!=0) exit(EXIT_SUCCESS);
	fh=open("/dev/null",O_RDWR,0666);
	close(0);
	close(1);
	close(2);
	dup2(fh, 0);
	dup2(fh, 1);
	dup2(fh, 2);
#ifdef HAVE_SETSID
	setsid();
#endif
}


/*
UNIX trivia - when is a problem a problem?  When you don't ignore it.
If you do nothing, well, you're not being ignorant enough.  You have
to be explicitly ignorant.
*/
void
ignorer (int x)
{
	signal (x, ignorer);
}

/*
 * TODO:
 * This is a potential deadlock condition.  If the thread handling
 * this signal holds the authdetails lock, it will attempt to take it
 * twice, and hold it forever; other threads will eventually try for
 * it and get caught in the traffic jam.
 *
 * I'm not sure what to do about it, tho. This signal can't be left to
 * one thread only because some OSes (thanks Linux!) send signals to
 * specific threads...
 *
 * Bottom line: don't SIGHUP unless you have to.  Ok?
 */
void reloadconfig(int x)
{
#ifdef WITH_DEBUG
	printf("%s: Configuration being reloaded\n",__FILE__);
#endif
	config.close();
	config.loadFile(sysconfdir "/antinat.conf");
	signal (x, reloadconfig);
}

void kidkiller(int x)
{
	int ret;
#ifdef WITH_DEBUG
	printf("%s: It was dead already, honest...\n",__FILE__);
#endif
	wait(&ret);
	signal(x, kidkiller);
}

void closeup(int x)
{
#ifdef WITH_DEBUG
	printf("%s: Closing up\n",__FILE__);
#endif
	logEvent(LOG_EVT_SERVERCLOSE);
	config.close();
	exit(EXIT_SUCCESS);
}

/*
main() - you've seen that before.
*/
int
main (int argc, char * argv[])
{
	DSParams param;
	DSComponent auth(&versions);
	DSComponent filter(&filters);
	DSComponent address(&addresses);
	DSComponent logs(&loggers);
	int i;
	unsigned long ip;
	signal (SIGCHLD, kidkiller);
	param.setFlagSwitch('x');
	param.process(argc,argv);
#ifndef WITH_DEBUG
	if (!param.getNumericValue('x')) daemonize();
#endif
	param.close();
	config.loadFile(sysconfdir "/antinat.conf");
	i=address.loadComponentsPrefixed(componentdir "/address", "address_type",
			"getfunction");
	if (i==0) {
		fprintf(stderr,"Could not load any address type handlers.  Antinat cannot continue.\n");
		return EXIT_FAILURE;
	}
#ifdef WITH_DEBUG
	printf("%s: Loaded %i address components\n",__FILE__,i);
#endif
	i=filter.loadComponentsPrefixed(componentdir "/filters", "filter_type",
			"filter");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i filter components\n",__FILE__,i);
#endif
	i=logs.loadComponentsPrefixed(componentdir "/loggers", "logger_type",
			"log");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i logger components\n",__FILE__,i);
#endif
	i=auth.loadComponentsPrefixed(componentdir "/versions", "version_type",
			"version_process");
	if (i==0) {
		fprintf(stderr,"Could not load any version handlers.  Antinat cannot continue.\n");
		return EXIT_FAILURE;
	}
#ifdef WITH_DEBUG
	printf("%s: Loaded %i version components\n",__FILE__,i);
#endif


	signal (SIGPIPE, ignorer);
	signal (SIGHUP, reloadconfig);
	signal (SIGQUIT, closeup);
	ip=inet_addr(config.getValue("boot","interface","0.0.0.0"));
	if (!server.startServer (config.getValueInt("boot","port",1080),ip))
	{
		printf ("%s: Could not initialise server.\n",__FILE__);
		return (EXIT_FAILURE);
	}
	logEvent(LOG_EVT_SERVERSTART);
	while (TRUE)
	{
		if (!HandleRequest ())
		{
			printf ("%s: Couldn't handle request.\n",__FILE__);
		}
	}
	return EXIT_SUCCESS;
}

