/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../../config.h"
#include <DSNetwork.h>
#include <DSConfigFile.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "../../Connection.h"
#include "../../Shared.h"
#include "../../loggers.h"

extern "C" {
int libv4bind_v4command_type()
{
	return 0x02;
}
}

/* This function reports to the client a socket's specs. */
void notifyclient(Connection * conn, SOCKET s,SOCKADDR_IN * sout, sl_t * soutlen)
{
	if (getsockname(s, (SOCKADDR *)sout, soutlen)==0) {
#ifdef WITH_DEBUG
		printf("%s: Bind succeeded, interface known\n",__FILE__);
#endif
		conn->sendResponsev4(90, sout->sin_addr.s_addr, sout->sin_port);
	} else {
#ifdef WITH_DEBUG
		printf("%s: Bind succeeded, interface unknown\n",__FILE__);
#endif
		conn->sendResponsev4(90, 0, 0);
	}
}

BOOL v4bind_command(Connection * conn)
{
	SOCKET remotelisten, remotetalk, top;
	SOCKADDR * addr;
	sl_t addrlen;
	SOCKADDR_IN sout;
	SOCKADDR_IN * inaddy;
	sl_t soutlen;
	fd_set fds;
	struct timeval tv;

	if (!conn->getDestSockaddr(&addr,&addrlen)) return FALSE;
	
#ifdef WITH_DEBUG
	printf("%s: Socket family: %x\n",__FILE__,addr->sa_family);
#endif
	remotelisten=socket(addr->sa_family, SOCK_STREAM, 0);
#ifdef WITH_DEBUG
	{
		SOCKADDR_IN * sin;
		sin=(SOCKADDR_IN *)addr;
		printf("%s: Binding to %x:%i\n",__FILE__,
				htonl(sin->sin_addr.s_addr),
				ntohs(sin->sin_port));
	}
#endif
	/* FIXME: This hack listens on all interfaces on any port.
	 * Is this really what the spec had in mind? */
	/* FIXME: SOCKSv4 explicitly says that the input interface
	 * sould be used to validate any incoming requests...ie.,
	 * this stuff is who we *want* to connect to us. */
	memset(&sout, 0, sizeof(sout));
	sout.sin_family=addr->sa_family;
	soutlen=sizeof(sout);
	if ((remotelisten<0)||
		(bind(remotelisten, (SOCKADDR *)&sout, soutlen)!=0)||
		(listen(remotelisten, 2)!=0)) {
#ifdef WITH_DEBUG
		printf("%s: Bind failed\n",__FILE__);
		perror("bind");
#endif
		CloseAndUnlink(remotelisten);
		conn->sendResponsev4(91, 0, 0);
		free(addr);
		return FALSE;
	}
	soutlen=sizeof(sout);
	notifyclient(conn, remotelisten, &sout, &soutlen);

	top=conn->getSocket();
	if (remotelisten>top) top=remotelisten;
	FD_ZERO(&fds);
	FD_SET(remotelisten,&fds);
	FD_SET(conn->getSocket(),&fds);
	tv.tv_sec=config.getValueInt("boot","maxbindwait",60);
	tv.tv_usec=0;
	select(top+1,&fds,NULL,NULL,&tv);
	if (FD_ISSET(remotelisten,&fds)) {
		/* Now to wait for incoming connection */
		remotetalk=accept(remotelisten, (SOCKADDR *)&sout, &soutlen);

		CloseAndUnlink(remotelisten);
		if (remotetalk<0) {
			/* Whoa, accept() failed.  Better tell client */
			conn->sendResponsev4(91, 0, 0);
			free(addr);
			return FALSE;
		}

		/* Verify incoming address */
		inaddy=(SOCKADDR_IN *)addr;
		/* Note: don't check ports... */
		if (!((inaddy->sin_family==sout.sin_family)&&
			((inaddy->sin_addr.s_addr==0)||(inaddy->sin_addr.s_addr==sout.sin_addr.s_addr)))) {
#ifdef WITH_DEBUG
			printf("%s: Addresses mismatch\n",__FILE__);
#endif
			conn->sendResponsev4(91, 0, 0);
			closesocket(remotetalk);
			free(addr);
			return FALSE;
		}
		free(addr);
		/* Tell client who just arrived */
		notifyclient(conn, remotetalk, &sout, &soutlen);

		conn->log(LOG_TYPE_CONNECTIONESTABLISHED);

		/* As with connect, start forwarding data. */
		conn->forwardData(remotetalk);
		closesocket(remotetalk);
	} else {
		CloseAndUnlink(remotelisten);
		conn->sendResponsev4(91, 0, 0);
		free(addr);
		return FALSE;
	}

	return TRUE;
}

extern "C" {
BOOL libv4bind_v4command(void * conn)
{
	return v4bind_command((Connection *)conn);
}
}


