/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../../config.h"
#include <DSNetwork.h>
#include <DSList.h>
#include <DSComponent.h>
#include "../ConnFive.h"
#include "../../moddecl.h"

DSHashTable *chaps;

static const char lets[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwyxz0123456789+/";

BOOL chap_gen_challenge(char * data, int len)
{
	int i;
	for (i=0;i<len;i++) {
		data[i]=lets[rand()%64];
	}
	return TRUE;
}

BOOL chap_send_fail(ConnFive * conn)
{
	char str[16];
	str[0]=0x01;
	str[1]=0x01;
	str[2]=0x00;
	str[3]=0x01;
	str[4]=0x01;
	return conn->sendData(str,5);
}

BOOL chap_send_ok(ConnFive * conn)
{
	char str[16];
	str[0]=0x01;
	str[1]=0x01;
	str[2]=0x00;
	str[3]=0x01;
	str[4]=0x00;
	return conn->sendData(str,5);
}

BOOL cpp_chap_auth(ConnFive * conn)
{
	proc_chap algo;
	unsigned char version;
	unsigned char tmpch;
	unsigned char navas;
	unsigned char att[10];
	char challenge[100];
	char username[256];
	int thealgo;
	unsigned char * correctresp;
	unsigned char * gotresp;
	char * passwd;
	int resplen;
	BOOL authed;
	BOOL finished;
	BOOL sentChallenge;
	BOOL sendChallenge;
	int sendAlgo;
	int i,j;
	finished=FALSE;

	authed=FALSE;
	gotresp=NULL;
	correctresp=NULL;
	passwd=NULL;
	sentChallenge=FALSE;
	thealgo=-1;
	strcpy(username,"");

	while (!finished) {
		sendChallenge=FALSE;
		sendAlgo=-1;
		if (!conn->getChar(&version)) return FALSE;
		if (version!=1) return FALSE;
		if (!conn->getChar(&navas)) return FALSE;
		printf("Chap: navas=%i\n",navas);
		for (i=0;i<navas;i++) {
			if (!conn->getSlab((char *)&att,2)) return FALSE;
			/* Documented:
			 * 0- status
			 * 1- Text-message
			 * 2- User-identity
			 * 3- Challenge
			 * 4- Response
			 * 5- Charset
			 * 16-Identifier
			 * 17-Algorithms
			*/
#ifdef WITH_DEBUG
			printf("%s: recved att %x data length %i\n",__FILE__,
				att[0], att[1]);
#endif
			switch (att[0]) {
				case 2:
					if (!conn->getSlab(username,att[1])) goto barfed;
#ifdef WITH_DEBUG
					printf("%s: Set username to %s\n",__FILE__,username);
#endif
					if (!sentChallenge) sendChallenge=TRUE;
					break;
				case 4:
					gotresp=(unsigned char *)malloc(att[1]+1);
					if (gotresp==NULL) {
						chap_send_fail(conn);
						goto barfed;
					}
					if (!conn->getSlab((char *)gotresp,att[1])) {
						chap_send_fail(conn);
						goto barfed;
					}
					if (thealgo<0) {
						chap_send_fail(conn);
						goto barfed;
					}
					algo=(proc_chap)chaps->getPtrValue(thealgo);
					if (algo==NULL) {
						/* Should never happen */
						chap_send_fail(conn);
						goto barfed;
					}
					if (!conn->getPasswordByUsername(username,&passwd)) {
						chap_send_fail(conn);
						goto barfed;
					}
					if (algo((unsigned char *)challenge,passwd,&correctresp,&resplen)==FALSE) {
						chap_send_fail(conn);
						goto barfed;
					}
					if (resplen!=att[1]) {
						/* If the lengths differ, so do the responses */
						chap_send_fail(conn);
						goto barfed;
					}
					if (memcmp(correctresp,gotresp,resplen)==0) {
						chap_send_ok(conn);
						conn->setUser(username);
						authed=TRUE;
						goto barfed;
					} else {
						chap_send_fail(conn);
						goto barfed;
					}
					
					
					
					break;
				case 0x11:
					for (j=0;j<att[1];j++) {
						if (!conn->getChar(&tmpch)) goto barfed;
						if (sendAlgo<0) {
							if (chaps->getPtrValue(tmpch)) sendAlgo=tmpch;
						}
						
					}
					if (sendAlgo<0) sendAlgo=-2;
					if (!sentChallenge) sendChallenge=TRUE;
					break;
			}
		}
		if (sendAlgo==-2) {
			chap_send_fail(conn);
			goto barfed;
		}
		att[0]=0x01;
		att[1]=0x00;
		if (sendChallenge) att[1]++;
		if (sendAlgo>=0) att[1]++;
		if (!conn->sendData((char *)att,2)) goto barfed;
		if (sendAlgo>=0) {
			att[0]=0x11;
			att[1]=0x01;
			att[2]=sendAlgo;
			if (!conn->sendData((char *)att,3)) goto barfed;
			thealgo=sendAlgo;
		}
		if (sendChallenge) {
			chap_gen_challenge(challenge,64);
			att[0]=0x03;
			att[1]=0x40;
			if (!conn->sendData((char *)att,2)) goto barfed;
			if (!conn->sendData((char *)challenge,64)) goto barfed;
		}
	}

	barfed:
	if (gotresp!=NULL)
		free(gotresp);
	if (correctresp!=NULL)
		free(correctresp);
	if (passwd!=NULL)
		free(passwd);
#ifdef WITH_DEBUG
	if (authed)
		printf("%s: authenticated successfully\n",__FILE__);
	else
		printf("%s: failed authentication\n",__FILE__);
#endif
	return authed;
}

extern "C" {
int libchap_authenticator_type()
{

	/* If there is no way of getting raw usernames and passwords,
	 * don't bother loading this thing, it will never work */

	if (v5getNumUnpwSrc()==0) return -1;


	int i;
	chaps=new DSHashTable();
	DSComponent ch(chaps);
	i=ch.loadComponentsPrefixed(componentdir "/version5/authenticators/chaps", "chap_type", "chap");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i chap handling components\n",__FILE__,i);
#endif
	return 3;
}

BOOL libchap_auth(void * conn)
{
	return cpp_chap_auth((ConnFive *)conn);
}
}
