/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../../config.h"
#include <DSNetwork.h>
#include <DSConfigFile.h>
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "../../Shared.h"
#include "../ConnFive.h"
#include "../../loggers.h"

extern "C" {
int libv5conn_v5command_type()
{
	return 0x01;
}
}

BOOL ver5conn_command(ConnFive * conn)
{
	SOCKET remote;
	PI_SA sout;
	SOCKADDR * addr;
	sl_t addrlen;
	sl_t soutlen;

	if (!conn->getDestSockaddr(&addr, &addrlen)) return FALSE;
#ifdef WITH_DEBUG
	printf("%s: Socket family: %x\n",__FILE__,addr->sa_family);
#endif
	remote=socket(addr->sa_family, SOCK_STREAM, 0);
#ifdef WITH_DEBUG
	{
		char * dest;
		dest=conn->getDestString();
		printf("%s: Connecting to %s\n",__FILE__,dest);
		free(dest);
	}
#endif

	if (connect(remote, addr, addrlen)!=0) {
#ifdef WITH_DEBUG
		printf("%s: Connection failed\n",__FILE__);
		perror("connect");
#endif
		closesocket(remote);
		conn->sendResponsev5(0x05);
		free(addr);
		return FALSE;
	}
	free(addr);
	soutlen=sizeof(sout);
	if (getsockname(remote, (SOCKADDR *)&sout, &soutlen)==0) {
#ifdef WITH_DEBUG
		printf("%s: Connection succeeded, interface known\n",__FILE__);
#endif
		conn->sendResponsev5(0x00, (SOCKADDR *)&sout);
	} else {
#ifdef WITH_DEBUG
		printf("%s: Connection succeeded, interface unknown\n",__FILE__);
#endif
		conn->sendResponsev5(0x00);
	}
	conn->log(LOG_TYPE_CONNECTIONESTABLISHED);
	conn->forwardData(remote);
	closesocket(remote);

	return TRUE;
}

extern "C" {
BOOL libv5conn_v5command(void * conn)
{
	return ver5conn_command((ConnFive *)conn);
}
}


