/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#undef _GNU_SOURCE
#define _GNU_SOURCE
#include "../../../config.h"
#include <DSMutex.h>
#ifdef HAVE_CRYPT_H
#include <crypt.h>
#endif
#ifdef HAVE_SHADOW_H
#include <shadow.h>
#endif
#ifdef HAVE_PWD_H
#include <pwd.h>
#endif
#include "../../Locks.h"
#include "../../Connection.h"

BOOL local_auth(char * username, char * passwd)
{
	char csalt[12];
	char * realpasswd;
	char * pwhash;
	BOOL authed;
#ifdef HAVE_GETSPNAM
	struct spwd * shad_pw;
#else
#ifdef HAVE_GETPWNAM
	struct passwd * pw_pw;
#endif
#endif
#if defined(HAVE_GETSPNAM_R)||(HAVE_GETPWNAM_R)
	char buf[1024];
#endif
	authed=FALSE;
	realpasswd=pwhash=NULL;
#ifdef HAVE_GETSPNAM    /* The system supports shadow password */
#ifndef HAVE_GETSPNAM_R /* System has no reentrant shadow functions */
	getspnam_lock.lock();
	shad_pw=getspnam(username);
#else                   /* System has reentrant shadow functions */
	struct spwd realspwd;
#ifdef BROKEN_GETSPNAM  /* System has Solaris-style shadow functions */
	shad_pw=getspnam_r(username, &realspwd, buf, sizeof(buf));
#else                   /* System has Linux-style shadow functions */
	getspnam_r(username, &realspwd, buf, sizeof(buf), &shad_pw);
#endif                  /* BROKEN_GETSPNAM */
#endif                  /* HAVE_GETSPNAM_R */
	if (shad_pw) {
		realpasswd=(char *)malloc(strlen(shad_pw->sp_pwdp)+1);
		strcpy(realpasswd,shad_pw->sp_pwdp);
	}
#ifndef HAVE_GETSPNAM_R /* No reentrant shadow functions */
	getspnam_lock.unlock();
#endif
#else                   /* No shadow - try passwd */
#ifdef HAVE_GETPWNAM    /* Do we have passwd functions */
#ifndef HAVE_GETPWNAM_R /* We have no reentrant passwd functions */
	getpwnam_lock.lock();
	pw_pw=getpwnam(username);
#else                   /* We have reentrant passwd functions */
	struct passwd realpwd;
#ifdef BROKEN_GETPWNAM  /* System has Solaris-style passwd functions */
	pw_pw=getpwnam_r(username, &realpwd, buf, sizeof(buf));
#else                   /* System has Linux-style passwd functions */
	getpwnam_r(username, &realpwd, buf, sizeof(buf), &pw_pw);
#endif                  /* BROKEN_GETPWNAM */
#endif                  /* HAVE_GETPWNAM_R */
	if (pw_pw) {
		realpasswd=(char *)malloc(strlen(pw_pw->pw_passwd)+1);
		strcpy(realpasswd,pw_pw->pw_passwd);
	}
#ifndef HAVE_GETPWNAM_R /* No reentrant passwd functions */
	getpwnam_lock.unlock();
#endif                  /* HAVE_GETPWNAM_R */
#endif                  /* HAVE_GETPWNAM */
#endif                  /* HAVE_GETSPNAM */
#ifdef WITH_DEBUG
	if (realpasswd) 
		printf("%s: Found user, has hash %s\n",__FILE__,realpasswd);
	else
		printf("%s: Username not found\n",__FILE__);
#endif
	if (realpasswd) {
		pwhash=(char *)malloc(40);
		csalt[0]=realpasswd[0];
		csalt[1]=realpasswd[1];
		csalt[2]='\0';
		if (strcmp(csalt,"$1")==0) {
			strncpy(csalt,realpasswd,11);
			csalt[12]='\0';
		}
		/* Apparently, crypt_r can't auth against non-MD5 passwds :( */
#if (defined(HAVE_CRYPT_CRYPT_R)||defined(HAVE_CRYPT_R))&&(!defined(BROKEN_CRYPT_R))
		struct crypt_data cd;
		strcpy(pwhash,crypt_r(passwd,csalt,&cd));
		if (strcmp(pwhash,realpasswd)==0) {
			authed=TRUE;
		} else {
#ifdef WITH_DEBUG
			printf("%s: Couldn't authenticate with crypt_r.  Trying crypt.\n",__FILE__);
#endif
#endif
			crypt_lock.lock();
			strcpy(pwhash,crypt(passwd,csalt));
			crypt_lock.unlock();
			if (strcmp(pwhash,realpasswd)==0) {
				authed=TRUE;
			}
#if (defined(HAVE_CRYPT_CRYPT_R)||defined(HAVE_CRYPT_R))&&(!defined(BROKEN_CRYPT_R))
		}
#endif
	}

	if (pwhash) free(pwhash);
	if (realpasswd) free(realpasswd);
	return authed;
}

extern "C" {
int liblocal_init_auth()
{
	return 0;
}

BOOL liblocal_auth(char * username, char * passwd)
{
	return local_auth(username,passwd);
}
}

