/* ANTINAT
 * =======
 * This software is Copyright (c) 2003 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../config.h"
#include <sys/types.h>
#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#include <DSNetwork.h>
#include <DSConfigFile.h>
#include <DSHashTable.h>
#include <DSComponent.h>
#include "../Connection.h"
#include "../Shared.h"
#include "../filters.h"
#include "../moddecl.h"

/* These structures should not be written to from within a connection thread,
 * only the master thread.  So they're not really protected. */
DSHashTable *v4_commands;


BOOL Ver4Request(Connection * conn)
{
	unsigned char cmd;
	unsigned char tmp;
	char temp[5];
	unsigned long ulAddr;
	proc_command command;

	if (!conn->getChar(&cmd)) return FALSE;

	conn->setDestAddressType(AF_INET);
	conn->getSlab(temp, 2);
	conn->setDestPort(*(unsigned short *)temp);
	conn->getSlab(temp, 4);
	ulAddr=ntohl(*(unsigned long *)temp);

	/* Read until receive a NULL and throw it away. */
	tmp=0x01;
	while (tmp!=0x00) {
		if (!conn->getChar(&tmp)) {
			return FALSE;
		}
	}

	if ((ulAddr>0)&&(ulAddr<256)) {
		/* This is a SOCKS 4a request.  It has a piggybacked name 
		 * trailing after the request body */
		char hostname[128];
		unsigned int i;
		BOOL bHaveHostname;
		bHaveHostname=FALSE;
		for (i=0;(i<sizeof(hostname)-1);i++) {
			if (!conn->getChar(&hostname[i])) {
				return FALSE;
			}
			if (hostname[i]=='\0') {
				bHaveHostname=TRUE;
				break;
			}
		}
		hostname[i]='\0';
#ifdef WITH_DEBUG
		printf("%s: Have hostname: %i, hostname: %s\n",__FILE__,
			bHaveHostname,hostname);
#endif
		if (!bHaveHostname) return FALSE;
		if (!conn->setDestHostname(hostname)) {
			conn->sendResponsev4(91, 0, 0);
			return FALSE;
		}

	} else {
		conn->setDestAddress((char *)temp,4);
	}

	if (!conn->IsAllowed(TYPE_DEST)) {
		return FALSE;
	}


	command=(proc_command)v4_commands->getPtrValue(cmd);
	if (command) {
#ifdef WITH_DEBUG
		printf("%s: Applying command\n",__FILE__);
#endif
		return command(conn);
	} else {
#ifdef WITH_DEBUG
		printf("%s: No command\n",__FILE__);
#endif
		conn->sendResponsev4(91, 0, 0);
		return FALSE;
	}
}

BOOL ver4_init_module()
{
	v4_commands=new DSHashTable();
	DSComponent com(v4_commands);
	int i;
	i=com.loadComponentsPrefixed(componentdir "/version4/commands",
			"v4command_type", "v4command");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i v4 command components\n",__FILE__,i);
#endif

	/* As with v5, this is a memleak, fix when libds lets us */
	if (i==0) return FALSE;
	return TRUE;
}

extern "C" {
int libv4_version_type()
{
	if (!ver4_init_module()) return -1;
	return 4;
}

BOOL libv4_version_process(void * conn)
{
	return Ver4Request((Connection *)conn);
}
}

