/* ANTINAT
 * =======
 * This software is Copyright (c) 2003-04 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#include "../../config.h"
#include <sys/types.h>
#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#include <DSNetwork.h>
#include <DSConfigFile.h>
#include <DSHashTable.h>
#include <DSComponent.h>
#include "../Shared.h"
#include "../version5/ConnFive.h"
#include "../filters.h"
#include "../moddecl.h"

/* These structures should not be written to from within a connection thread,
 * only the master thread.  So they're not really protected. */
DSHashTable *authenticators;
DSHashTable *v5_commands;
DSList *unpwsrc;
DSList *unpwauth;
DSHashTable *resolvers;
DSHashTable *revresolve;

int nResolvers=0;
int nRevRes=0;
int nUnpwSrc=0;
int nUnpwAuth=0;
int nAuthenticators=0;
int nCommands=0;

BOOL Initialize(ConnFive * conn)
{
	unsigned char nMethods;
	unsigned char method;
	int i;
	proc_auth processor;
	void * tmp;
	unsigned char chosenmethod;
	char szResp[2];
	processor=NULL;
	chosenmethod=0xff;

	if (!conn->getChar(&nMethods)) return FALSE;
#ifdef WITH_DEBUG
	printf("%s: Number of Methods: %i\n",__FILE__,nMethods);
#endif
	for (i=0;i<nMethods;i++) {
		if (!conn->getChar(&method)) return FALSE;
#ifdef WITH_DEBUG
		printf("%s: Method %i: %x\n",__FILE__,i,method);
#endif
		tmp=authenticators->getPtrValue(method);
		if (tmp&&(!processor)) {
			processor=(proc_auth)tmp;
			chosenmethod=method;
#ifdef WITH_DEBUG
			printf("%s: Chosen method %i\n",__FILE__,method);
#endif
		}
	}
	szResp[0]=0x05;
	szResp[1]=chosenmethod;
	conn->sendData(szResp,2);
	if (chosenmethod!=0xff) {
		return processor(conn);
	} else return FALSE;
}

BOOL Ver5Request(ConnFive * conn)
{
	unsigned char version;
	unsigned char cmd;
	unsigned char res;
	unsigned char atyp;
	void * tmp;
	proc_resolve resolver;

	if (!conn->getChar(&version)) return FALSE;
	if (version!=5) return FALSE;
	if (!conn->getChar(&cmd)) return FALSE;
	if (!conn->getChar(&res)) return FALSE;
	if (!conn->getChar(&atyp)) return FALSE;
	resolver=conn->getResolver(atyp);
	if (resolver) {
#ifdef WITH_DEBUG
		printf("%s: Applying name resolver %x\n",__FILE__,atyp);
#endif
		resolver(conn);
	} else {
#ifdef WITH_DEBUG
		printf("%s: No resolver\n",__FILE__);
#endif 
		conn->sendResponsev5(0x08);
		return FALSE;
	}
	if (conn->getDestAddressLength()==0) {
#ifdef WITH_DEBUG
		printf("%s: Resolver failed\n",__FILE__);
#endif 
		conn->sendResponsev5(0x04); /* Host unreachable */
		return FALSE;
	}
	if (!conn->IsAllowed(TYPE_DEST)) {
		return FALSE;
	}
	tmp=v5_commands->getPtrValue(cmd);
	if (tmp) {
		BOOL ret;
		proc_command command;
#ifdef WITH_DEBUG
		printf("%s: Applying command\n",__FILE__);
#endif
		command=(proc_command)tmp;
		ret=command(conn);
		return ret;
	} else {
#ifdef WITH_DEBUG
		printf("%s: No command %x\n",__FILE__,cmd);
#endif
		conn->sendResponsev5(0x07);
		return FALSE;
	}
	return FALSE;
}

BOOL cpp_version_process(ConnFive * conn)
{
	if (!Initialize(conn)) return FALSE;
#ifdef WITH_DEBUG
	printf("%s: passed initialization\n",__FILE__);
#endif
	return Ver5Request(conn);
}

BOOL ver5_init_module()
{
	authenticators=new DSHashTable();
	v5_commands=new DSHashTable();
	unpwsrc=new DSList();
	unpwauth=new DSList();
	resolvers=new DSHashTable();
	revresolve=new DSHashTable();
	DSComponent auth(authenticators);
	DSComponent com(v5_commands);
	DSComponent com_unpwsrc(unpwsrc);
	DSComponent com_unpwauth(unpwauth);
	DSComponent res(resolvers);
	DSComponent revres(revresolve);

	/* The order here is things least likely to have dependencies to
	 * those that do. */

	nResolvers=res.loadComponentsPrefixed(
			componentdir "/version5/resolvers", "resolver_type",
			"resolve");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i resolver components\n",__FILE__,nResolvers);
#endif
	nRevRes=revres.loadComponentsPrefixed(componentdir "/version5/revres",
			"revres_type", "revres");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i reverse resolver components\n",__FILE__,nRevRes);
#endif


	nUnpwSrc=com_unpwsrc.loadComponentsPrefixed(
			componentdir "/version5/unpwsrc", "init_src", "getpw");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i v5 username-password sources\n",__FILE__,nUnpwSrc);
#endif
	nUnpwAuth=com_unpwauth.loadComponentsPrefixed(
			componentdir "/version5/unpwauth", "init_auth", "auth");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i v5 username-password authenticators\n",__FILE__,nUnpwAuth);
#endif


	nAuthenticators=auth.loadComponentsPrefixed(
			componentdir "/version5/authenticators",
			"authenticator_type", "auth");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i authenticator components\n",
		__FILE__,nAuthenticators);
#endif
	nCommands=com.loadComponentsPrefixed(componentdir "/version5/commands",
			"v5command_type", "v5command");
#ifdef WITH_DEBUG
	printf("%s: Loaded %i v5 command components\n",__FILE__,nCommands);
#endif

	/* FIXME: Allow unloading of components on failure when libds supports
	 * it - at the moment, this is a memory leak. */

	/* If we don't have one Command handler, one Authenticator, and
	 * one Resolver (forward and reverse), libv5 can't work. */
	if (nCommands&&nAuthenticators&&nResolvers&&nRevRes) return TRUE;
	return FALSE;
}

extern "C" {
int libv5_version_type()
{
	if (!ver5_init_module()) return -1;
	return 5;
}

BOOL libv5_version_process(void * conn)
{
	return cpp_version_process((ConnFive *)conn);
}
}

/** OKAY: Module stuff over.  This is functionality this module provides
 * to other submodules, in order to keep it out of core. */

ConnFive::ConnFive (DSNetwork * thenet) : Connection(thenet)
{
}

ConnFive::ConnFive (SOCKADDR * sa, char * udppacket, int udplen):
	Connection (sa, udppacket, udplen)
{
}

BOOL ConnFive::getPasswordByUsername(const char * username, char ** passwd)
{
	DSListElement * le;
	proc_unpwsrc src;
	for (le=unpwsrc->getHead();le!=NULL;le=le->getNext()) {
		src=(proc_unpwsrc)le->getDataPtr();
		if (src(username, passwd))
			return TRUE;
	}
	return FALSE;
}

BOOL ConnFive::authUsernamePassword(const char * username, const char * passwd)
{
	DSListElement * le;
	proc_unpwauth auther;
	char * correctpw;
	for (le=unpwauth->getHead();le!=NULL;le=le->getNext()) {
		auther=(proc_unpwauth)le->getDataPtr();
		if (auther(username, passwd))
			return TRUE;
	}

	if (getPasswordByUsername(username, &correctpw)) {
		if (strcmp(passwd,correctpw)==0) {
			free(correctpw);
			return TRUE;
		}
		free(correctpw);
	}
	return FALSE;
}

BOOL ConnFive::sendResponsev5(char rep, SOCKADDR * sa)
{
	char szResp[100];
	char szTemp[10];
	char * oldbuffer;
	int oldbuflen;
	int oldbuff_upto;
	int havetosend;
	proc_revresolve revres;

	/* Take a copy of the old buffer info */
	oldbuffer=buffer;
	oldbuflen=bufflen;
	oldbuff_upto=buff_upto;

	/* Set up writes to temp location */
	buffer=szResp;
	bufflen=sizeof(szResp);
	buff_upto=0;

	revres=NULL;
	szTemp[0]=0x05;
	szTemp[1]=rep;
	szTemp[2]=0x00;
	sendData(szTemp,3);
	if (sa)
		revres=(proc_revresolve)getRevResolver(sa->sa_family);

	if (revres==NULL) {
		int i;
		szTemp[0]=0x01;
		for (i=1;i<7;i++) szTemp[i]=0x00;
		sendData(szTemp,7);
	} else {
		revres(this, sa);
	}

	havetosend=buff_upto;

	/* Write to wherever the old buffer was, hopefully network */
	buffer=oldbuffer;
	bufflen=oldbuflen;
	buff_upto=oldbuff_upto;

	sendData(szResp, havetosend);
	return TRUE;
}

proc_resolve ConnFive::getResolver(int nResolver)
{
	return (proc_resolve)resolvers->getPtrValue(nResolver);
}

proc_revresolve ConnFive::getRevResolver(int nResolver)
{
	return (proc_revresolve)revresolve->getPtrValue(nResolver);
}

int v5getNumResolvers()
{
	return nResolvers;
}

int v5getNumRevResolvers()
{
	return nRevRes;
}

int v5getNumUnpwAuth()
{
	return nUnpwAuth;
}

int v5getNumUnpwSrc()
{
	return nUnpwSrc;
}

int v5getNumAuthenticators()
{
	return nAuthenticators;
}

int v5getNumCommands()
{
	return nCommands;
}

